@@ -32,7 +32,7 @@ from .line import Line
 from .layout import Layout
 from .layout.prompt import DefaultPrompt
 from .renderer import Renderer
-from .utils import raw_mode, cooked_mode, call_on_sigwinch
+from .utils import raw_mode, cooked_mode, call_on_sigwinch, EventHook
 from .history import History
 
 from pygments.styles.default import DefaultStyle
@@ -87,11 +87,11 @@ class CommandLineInterface(object):
     def __init__(self, stdin=None, stdout=None,
                  layout=None,
                  line=None,
-                 renderer_factory=Renderer,
                  default_input_mode=InputMode.INSERT,
                  style=DefaultStyle,
                  key_binding_factories=None,
-                 async_autocomplete_on_text_insert=False):
+                 create_async_autocompleters=True,
+                 renderer_factory=Renderer):
 
         self.stdin = stdin or sys.stdin
         self.stdout = stdout or sys.stdout
@@ -137,12 +137,24 @@ class CommandLineInterface(object):
         self._calls_from_executor = []
 
         # Handle events.
-        if async_autocomplete_on_text_insert:
-            self.lines['default'].onTextInsert += self._async_autocomplete
-        self.complete_thread_running = False
+        if create_async_autocompleters:
+            for n, l in self.lines.items():
+                if l.completer:
+                    l.onTextInsert += self._create_async_complter(n)
 
         self._reset()
 
+        # Events
+
+        #: Called when there is no input for x seconds.
+        #:   At this event, you can for instance start a background thread to
+        #:   generate information about the input. E.g. get the code signature
+        #:   of the function below the cursor position in the case of a REPL.
+        self.onInputTimeout = EventHook()
+
+        self.onReadInputStart = EventHook()
+        self.onReadInputEnd = EventHook()
+
     @property
     def line(self):
         return self.lines['default']
@@ -199,21 +211,6 @@ class CommandLineInterface(object):
         if self._schedule_pipe:
             os.write(self._schedule_pipe[1], b'x')
 
-    def on_input_timeout(self):
-        """
-        Called when there is no input for x seconds.
-        """
-        # At this place, you can for instance start a background thread to
-        # generate information about the input. E.g. the code signature of the
-        # function below the cursor position in the case of a REPL.
-        pass
-
-    def on_read_input_start(self):  # XXX: replace with EventHook
-        pass
-
-    def on_read_input_end(self):  # XXX: replace with EventHook
-        pass
-
     def _get_char_loop(self):
         """
         The input 'event loop'.
@@ -239,8 +236,8 @@ class CommandLineInterface(object):
                 for c in calls_from_executor:
                     c()
             else:
-                # Handle input timeout.
-                self.on_input_timeout()
+                # Fire input timeout event.
+                self.onInputTimeout.fire()
                 timeout = None
 
     def _read_from_stdin(self):
@@ -302,8 +299,8 @@ class CommandLineInterface(object):
                 self.layout.reset()
             reset_line()
 
-            # Trigger read_start.
-            self.on_read_input_start()
+            # Trigger onReadInputStart event.
+            self.onReadInputStart.fire()
 
             with raw_mode(self.stdin.fileno()):
                 self.inputstream.prepare_terminal(self.stdout)
@@ -371,8 +368,8 @@ class CommandLineInterface(object):
                 os.close(schedule_pipe[0])
                 os.close(schedule_pipe[1])
 
-            # Trigger read_end.
-            self.on_read_input_end()
+            # Trigger onReadInputEnd event.
+            self.onReadInputEnd.fire()
 
             self.is_reading_input = False
 
@@ -412,52 +409,57 @@ class CommandLineInterface(object):
     def is_returning(self):
         return self._return_code
 
-    def _async_autocomplete(self):
+    def _create_async_complter(self, line_name):
         """
-        Asynchronous autocompletion while typing.
+        Create function for asynchronous autocompletion while typing.
         (Autocomplete in other thread.)
         """
-        document = self.line.document
+        line = self.lines[line_name]
+        complete_thread_running = [False] # By ref.
 
-        # Don't start two threads at the same time.
-        if self.complete_thread_running:
-            return
+        def async_completer():
+            document = line.document
 
-        # Don't complete when we already have completions.
-        if self.line.complete_state:
-            return
+            # Don't start two threads at the same time.
+            if complete_thread_running[0]:
+                return
 
-        # Don't automatically complete on empty inputs.
-        char = document.char_before_cursor
-        if not self.line.text or char.isspace():
-            return
+            # Don't complete when we already have completions.
+            if line.complete_state:
+                return
 
-        # Otherwise, get completions in other thread.
-        self.complete_thread_running = True
+            # Don't automatically complete on empty inputs.
+            char = document.char_before_cursor
+            if not line.text or char.isspace():
+                return
 
-        def run():
-            completions = list(self.line.completer.get_completions(document))
-            self.complete_thread_running = False
+            # Otherwise, get completions in other thread.
+            complete_thread_running[0] = True
 
-            def callback():
-                """
-                Set the new complete_state in a safe way. Don't replace an
-                existing complete_state if we had one. (The user could have
-                pressed 'Tab' in the meantime. Also don't set it if the text
-                was changed in the meantime.
-                """
-                # Set completions if the text was not yet changed.
-                if self.line.text == document.text and \
-                        self.line.cursor_position == document.cursor_position and \
-                        not self.line.complete_state:
-                    self.line._start_complete(go_to_first=False, completions=completions)
-                    self._redraw()
-                else:
-                    # Otherwise, restart thread.
-                    self._async_autocomplete()
-            self.call_from_executor(callback)
+            def run():
+                completions = list(line.completer.get_completions(document))
+                complete_thread_running[0] = False
 
-        self.run_in_executor(run)
+                def callback():
+                    """
+                    Set the new complete_state in a safe way. Don't replace an
+                    existing complete_state if we had one. (The user could have
+                    pressed 'Tab' in the meantime. Also don't set it if the text
+                    was changed in the meantime.
+                    """
+                    # Set completions if the text was not yet changed.
+                    if line.text == document.text and \
+                            line.cursor_position == document.cursor_position and \
+                            not line.complete_state:
+                        line._start_complete(go_to_first=False, completions=completions)
+                        self._redraw()
+                    else:
+                        # Otherwise, restart thread.
+                        async_completer()
+                self.call_from_executor(callback)
+
+            self.run_in_executor(run)
+        return async_completer
 
 
 def _select(*args, **kwargs):
