@@ -101,6 +101,8 @@ class PythonStyle(Style):
         Token.Prompt.Arg:              'noinherit',
         Token.Prompt.Arg.Text:          'bold',
 
+        Token.Layout.Toolbar.System.Prefix: '#44ffff noinherit',
+
         Token.Toolbar:                 'bg:#222222 #aaaaaa',
         Token.Toolbar.Off:             'bg:#222222 #888888',
         Token.Toolbar.On:              'bg:#222222 #ffffff',
@@ -179,7 +181,7 @@ def python_bindings(registry, cli_ref):
             """
             Enable/Disable multiline mode.
             """
-            line.is_multiline = not line.is_multiline
+            line.always_multiline = not line.always_multiline
 
     @handle(Keys.Tab, in_mode=InputMode.INSERT)
     def _(event):
@@ -515,45 +517,47 @@ class PythonCommandLineInterface(CommandLineInterface):
             style=style,
             key_binding_factories=key_binding_factories,
             line=line,
-            async_autocomplete_on_text_insert=True)
+            create_async_autocompleters=True)
 
-    def on_input_timeout(self):
-        """
-        When there is no input activity,
-        in another thread, get the signature of the current code.
-        """
-        # Never run multiple get-signature threads.
-        if self.get_signatures_thread_running:
-            return
-        self.get_signatures_thread_running = True
+        def on_input_timeout():
+            """
+            When there is no input activity,
+            in another thread, get the signature of the current code.
+            """
+            # Never run multiple get-signature threads.
+            if self.get_signatures_thread_running:
+                return
+            self.get_signatures_thread_running = True
 
-        document = self.line.document
+            document = self.line.document
 
-        def run():
-            script = get_jedi_script_from_document(document, self.locals, self.globals)
+            def run():
+                script = get_jedi_script_from_document(document, self.locals, self.globals)
 
-            # Show signatures in help text.
-            if script:
-                try:
-                    signatures = script.call_signatures()
-                except ValueError:
-                    # e.g. in case of an invalid \\x escape.
+                # Show signatures in help text.
+                if script:
+                    try:
+                        signatures = script.call_signatures()
+                    except ValueError:
+                        # e.g. in case of an invalid \\x escape.
+                        signatures = []
+                    except Exception:
+                        # Sometimes we still get an exception (TypeError), because
+                        # of probably bugs in jedi. We can silence them.
+                        signatures = []
+                else:
                     signatures = []
-                except Exception:
-                    # Sometimes we still get an exception (TypeError), because
-                    # of probably bugs in jedi. We can silence them.
-                    signatures = []
-            else:
-                signatures = []
 
-            self.get_signatures_thread_running = False
+                self.get_signatures_thread_running = False
 
-            # Set signatures and redraw if the text didn't change in the
-            # meantime. Otherwise request new signatures.
-            if self.line.text == document.text:
-                self.line.signatures = signatures
-                self.request_redraw()
-            else:
-                self.on_input_timeout()
+                # Set signatures and redraw if the text didn't change in the
+                # meantime. Otherwise request new signatures.
+                if self.line.text == document.text:
+                    self.line.signatures = signatures
+                    self.request_redraw()
+                else:
+                    self.on_input_timeout()
 
-        self.run_in_executor(run)
+            self.run_in_executor(run)
+
+        self.onInputTimeout += on_input_timeout
