@@ -104,7 +104,7 @@ class Line(object):
     also implements the history, undo stack, reverse search and the completion
     state.
 
-    :attr completer : :class:`~prompt_toolkit.completion.Completer` class.
+    :attr completer : :class:`~prompt_toolkit.completion.Completer` instance.
     :attr history: :class:`~prompt_toolkit.history.History` instance.
     :attr callbacks: :class:`~.Callbacks` instance.
 
@@ -130,13 +130,13 @@ class Line(object):
 
         self.__cursor_position = 0
 
-        self.reset()
-
         # Events
         self.onTextChanged = EventHook()
         self.onTextInsert = EventHook()
         self.onCursorPositionChanged = EventHook()
 
+        self.reset()
+
     def reset(self, initial_value=''):
         self.cursor_position = len(initial_value)
 
@@ -194,6 +194,9 @@ class Line(object):
         # Note that the cursor position can change if we have a selection the
         # new position of the cursor determines the end of the selection.
 
+        # fire 'onCursorPositionChanged' event.
+        self.onCursorPositionChanged.fire()
+
     @property
     def working_index(self):
         return self.__working_index
@@ -581,8 +584,15 @@ class Line(object):
         if not self.isearch_state:
             self.start_isearch()
 
-        # When `text` is not a suffix of the search string that we had.
-        if not text.startswith(self.isearch_state.isearch_text):
+        # When backspace has been pressed.
+        if self.isearch_state.no_match_from_index and \
+                self.isearch_state.isearch_text.startswith(text):
+            if len(text) < self.isearch_state.no_match_from_index:
+                self.isearch_state.no_match_from_index = None
+
+        # When not appending text.
+        # (When `text` is not a suffix of the search string that we had.)
+        elif not text.startswith(self.isearch_state.isearch_text):
             self.isearch_state.no_match_from_index = None
 
             self.working_index = self.isearch_state.original_working_index
@@ -776,7 +786,9 @@ class Line(object):
         self.selection_state = None
 
     def open_in_editor(self):
-        """ Open code in editor. """
+        """
+        Open code in editor.
+        """
         # Write to temporary file
         descriptor, filename = tempfile.mkstemp(self.tempfile_suffix)
         os.write(descriptor, self.text.encode('utf-8'))
